' Some BASIC languages have a SHIFTIN and SHIFTOUT command.
' 
' These sub-procedures replace the need for these commands with PICAXE.
' To use, simply copy the symbol definitions to the top of your program
' and copy the appropriate shiftin / shiftout sub procedures to the 
' bottom of your program.
' Do not copy all options as this will waste memory space.

' If the gosub is only used once, consider moving the code within the sub
' into your main program (removing the gosub and return commands) to 
' save space on the PICAXE gosub stack.

' It is presumed that the data and clock outputs (sdata and sclk)
' are in the low condition before the gosub is used.

' e.g. BASIC line
'                   shiftin serdata, sclk, mode, (var_in(\bits))
' becomes
'
' 	gosub shiftin_LSB_Pre		(for mode LSBPre)
' or	gosub shiftin_MSB_Pre		(for mode MSBPre)
' or	gosub shiftin_LSB_Post		(for mode LSBPost)
' or	gosub shiftin_MSB_Post		(for mode MSBPost)
'
' and BASIC line
'                   shiftout sdata, sclk, mode, (var_out(\bits))
' becomes
'
' 	gosub shiftout_LSBFirst		(for mode LSBFirst)
' or	gosub shiftout_MSBFirst		(for mode MSBFirst)



' ~~~~~ SYMBOL DEFINITIONS ~~~~~
' Required for all routines. Change pin numbers and bits as required.
' Uses variables b7-b13 (i.e. b7,w4,w5,w6). If only using 8 bits
' all the word variables can be safely changed to byte variables.

' ========================================================================
' ***** Sample symbol definitions *****
symbol sclk = 5		' clock (output pin)
symbol sdata = 7	' data (output pin for shiftout)
symbol serdata = input7	' data (input pin for shiftin, note input7 *NOT* 7)

symbol counter = b7	' variable used during loop
symbol mask = w4 	' bit masking variable
symbol var_in = w5	' data variable used durig shiftin
symbol var_out = w6	' data variable used during shiftout

symbol bits = 8		' number of bits
symbol MSBvalue = 128	' MSBvalue (=128 for 8 bits, 512 for 10 bits, 2048 for 12 bits)

' ========================================================================


' ~~~~~ SHIFTIN ROUTINES ~~~~~
' Only one of these 4 is required - see your IC requirements
' It is recommended you delete the others to save space


' ========================================================================
' ***** Shiftin LSB first, Data Pre-Clock *****
' Shift in the data LSB first into variable var_in 
' Read data before sending clock pulse 
' Using clock output pin sclk
' Using data input pin serdata 


shiftin_LSB_Pre:
	let var_in = 0
	for counter = 1 to bits			' number of bits
		var_in = var_in / 2 		' shift right as LSB first	
		if serdata = 0 then skipLSBPre
		var_in = var_in + MSBValue	' set MSB bit if serdata = 1
skipLSBPre:
		pulsout sclk,1 			' pulse clock to get next data bit
	next counter
	return
' ========================================================================

' ========================================================================
' ***** Shiftin MSB first, Data Pre-Clock *****
' Shift in the data MSB first into variable var_in 
' Read data before sending clock pulse 
' Using clock output pin sclk
' Using data input pin serdata 

shiftin_MSB_Pre:
	let var_in = 0
	for counter = 1 to bits			' number of bits
		var_in = var_in * 2 		' shift left as MSB first
		if serdata = 0 then skipMSBPre
		var_in = var_in + 1 		' set LSB if serdata = 1
skipMSBPre:
		pulsout sclk,1 			' pulse clock to get next data bit
	next counter
	return
' ========================================================================

' ========================================================================
' ***** Shiftin LSB first, Data Post-Clock *****
' Shift in the data LSB first into variable var_in 
' Read data after sending clock pulse 
' Using clock output pin sclk
' Using data input pin serdata 


shiftin_LSB_Post:
	let var_in = 0
	for counter = 1 to bits			' number of bits
		var_in = var_in / 2 		' shift right as LSB first	
		pulsout sclk,1 			' pulse clock to get next data bit
		if serdata = 0 then skipLSBPost
		var_in = var_in + MSBValue	' set MSB bit if serdata = 1 
skipLSBPost:
	next counter
	return
' ========================================================================

' ========================================================================
' ***** Shiftin MSB first, Data Post-Clock *****
' Shift in the data MSB first into variable var_in 
' Read data after sending clock pulse 
' Using clock output pin sclk
' Using data input pin serdata 

shiftin_MSB_Post:
	let var_in = 0
	for counter = 1 to bits			' number of bits
		var_in = var_in * 2 		' shift left as MSB first
		pulsout sclk,1 			' pulse clock to get next data bit
		if serdata = 0 then skipMSBPost
		var_in = var_in + 1 		' set LSB if serdata = 1
skipMSBPost:
	next counter
	return
' ========================================================================




' ~~~~~ SHIFTOUT ROUTINES ~~~~~
' Only one of these 2 is required - see your IC requirements
' It is recommended you delete the others to save space


' ========================================================================
' ***** Shiftout LSB first *****
' Shift out the data LSB first from variable var_out 
' Using clock output pin sclk
' Using data output pin sdata 

shiftout_LSBFirst:
	for counter = 1 to bits			' number of bits
		mask = var_out & 1 		' mask LSB 
		low sdata 				' data low
		if mask = 0 then skipLSB  
		high sdata				' data high
skipLSB:
		pulsout sclk,1 			' pulse clock for 10us 
		var_out = var_out / 2 		' shift variable right for LSB 
	next counter
	return
' ========================================================================

' ========================================================================
' ***** Shiftout MSB first *****
' Shift out the data MSB first from variable var_out 
' Using clock output pin sclk
' Using data output pin sdata 

' Note the number 128 (used twice) is the mask byte for MSB of 8 bits
' If using 10 bits use 512, 12 bits use 2048 etc

shiftout_MSBFirst:
	for counter = 1 to bits			' number of bits
		mask = var_out & MSBValue 	' mask MSB 
		high sdata 				' data high
		if mask = MSBValue then skipMSB  	
		low sdata				' data low
skipMSB:
		pulsout sclk,1 			' pulse clock for 10us 
		var_out = var_out * 2 		' shift variable left for MSB 
	next counter
	return
' ========================================================================

